#!/usr/bin/env python3
"""
NetBIOS Name Scanner
Parameters:
    - IP: Target IP or network range (e.g., 192.168.1.0/24)
"""
import socket
import struct
import ipaddress
import concurrent.futures
import argparse
from datetime import datetime

class NetBIOSScanner:
    def __init__(self):
        self.NETBIOS_PORT = 137
        self.NETBIOS_TIMEOUT = 2
        
    def create_name_query(self):
        """Create a NetBIOS name query packet"""
        transaction_id = b'\x00\x00'
        flags = b'\x01\x10'  # Standard query, recursion desired
        questions = b'\x00\x01'  # One question
        answers = b'\x00\x00'
        authority = b'\x00\x00'
        additional = b'\x00\x00'
        
        # Query for wildcard name
        name = b'*' + b'\x00' * 15  # Pad to 16 bytes
        name = bytes([32]) + name  # Length byte
        
        query_type = b'\x00\x21'  # NBSTAT query
        query_class = b'\x00\x01'  # IN class
        
        return (transaction_id + flags + questions + answers +
                authority + additional + name + query_type + query_class)
    
    def parse_response(self, data):
        """Parse NetBIOS name response"""
        try:
            names = []
            # Skip header (12 bytes) and question section
            offset = 12 + 34
            
            # Get number of names
            num_names = data[offset]
            offset += 1
            
            # Parse each name
            for _ in range(num_names):
                name = data[offset:offset+15].decode('ascii').rstrip()
                flags = struct.unpack('!H', data[offset+15:offset+17])[0]
                
                # Determine name type
                name_type = flags & 0x1F
                if name_type == 0x00:
                    name_type = "Workstation"
                elif name_type == 0x03:
                    name_type = "Messenger Service"
                elif name_type == 0x20:
                    name_type = "File Server"
                else:
                    name_type = f"Type 0x{name_type:02x}"
                
                names.append((name, name_type))
                offset += 18
                
            return names
            
        except Exception:
            return []
    
    def query_host(self, ip):
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.settimeout(self.NETBIOS_TIMEOUT)
            
            # Send query
            sock.sendto(self.create_name_query(), (str(ip), self.NETBIOS_PORT))
            
            # Get response
            data, _ = sock.recvfrom(1024)
            names = self.parse_response(data)
            
            if names:
                print(f"\nHost: {ip}")
                for name, name_type in names:
                    print(f"  {name}: {name_type}")
                print("-" * 40)
                
            sock.close()
            return True
            
        except socket.timeout:
            return False
        except Exception as e:
            print(f"Error querying {ip}: {str(e)}")
            return False
    
    def scan_network(self, network):
        try:
            network = ipaddress.ip_network(network)
            print(f"Starting NetBIOS scan on network {network}")
            print(f"Timestamp: {datetime.now()}\n")
            
            hosts_found = 0
            with concurrent.futures.ThreadPoolExecutor(max_workers=50) as executor:
                futures = [executor.submit(self.query_host, ip) for ip in network.hosts()]
                for future in concurrent.futures.as_completed(futures):
                    if future.result():
                        hosts_found += 1
            
            print(f"\nScan complete. Found {hosts_found} hosts with NetBIOS names.")
            
        except Exception as e:
            print(f"Error during scan: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="NetBIOS Name Scanner")
    parser.add_argument("network", help="Target IP or network range (e.g., 192.168.1.0/24)")
    args = parser.parse_args()
    
    scanner = NetBIOSScanner()
    scanner.scan_network(args.network)
